import argparse, sys
from urllib.error import URLError
from urllib.parse import urlparse,urljoin
from urllib.request import urlopen
from html.parser import HTMLParser
import gzip
import zlib
from io import BytesIO

def validate_url(url):
	p = urlparse(url)
	return True if all([p.scheme, p.netloc]) and p.scheme != 'mailto' else False

class LinkParser(HTMLParser):
	def __init__(self):
		HTMLParser.__init__(self)
		self.links = []

	def handle_starttag(self, tag, attrs):
		if tag != 'a':
			return
		attr = dict(attrs)
		try:
			self.links.append(attr["href"])
		except KeyError:
			return

	def feed(self, data):
		self.output = []
		HTMLParser.feed(self, data)
		return self.links

def decode_content_body(data, encoding):
	if encoding == 'identity':
		text = data
	elif encoding in ('gzip', 'x-gzip'):
		io = BytesIO(data)
		with gzip.GzipFile(fileobj=io) as f:
			text = f.read()
	elif encoding == 'deflate':
		try:
			text = zlib.decompress(data)
		except zlib.error:
			text = zlib.decompress(data, -zlib.MAX_WBITS)
	else:
		return None
	
	return text

def process_url(url):
	parser = LinkParser()

	try:
		res = urlopen(url)
	except URLError:
		return None

	content_encoding = res.headers.get('Content-Encoding', 'identity')
	res_body_plain = decode_content_body(res.read(), content_encoding)

	if not res_body_plain:
		return None
	
	links = parser.feed(res_body_plain.decode("utf-8"))


	for i in range(len(links)):
		if links[i] is None:
			del links[i]
		if validate_url(links[i]) == False:
			links[i] = urljoin(url, links[i])

	return links


def main(args):

	visited = []

	nodes={}

	links = process_url(args.hostname)

	total = int(args.n);

	while (total and links):
		link = links.pop(0)
		total-=1

		cur_netloc = urlparse(link)[1]

		if cur_netloc and cur_netloc not in nodes:
			nodes[cur_netloc] = []

		print("Visiting #{}: {}".format(total,link))
		if (link not in visited):
			visited.append(link)
			more_links = process_url(link)
			if more_links is not None:
				print("Urls {} found. The queue is now: {}".format(len(more_links),len(links)))
				for new_link in more_links:
					if new_link not in visited:
						links.append(new_link)
						new_netloc = (urlparse(new_link)[1])
						if new_netloc and new_netloc != cur_netloc and new_netloc not in nodes[cur_netloc]:
							nodes[cur_netloc].append(new_netloc)

	# Dot file
	f = open('network_graph.dot', 'w')
	f.write('digraph G {\n')
	f.write('graph [splines=ortho, nodesep=1, rankdir = "TB"]\n')
	for node,connected in nodes.items():
		for c in connected:
				if c in nodes:
					if node in nodes[c]:
						f.write('"{}" -> "{}";\n'.format(node,c))
	f.write('}\n')
	f.close()

	print(total)
	return

if __name__ == '__main__':
	parser = argparse.ArgumentParser(description = 'Network Graph')
	parser.add_argument('hostname', help='The website URL from which to spidering.')
	parser.add_argument('-n',  metavar='N',  type=int, help='Number of urls to parse from the queue',default=200)
	args = parser.parse_args()

	if validate_url(args.hostname) == False:
		parser.print_help()
		sys.exit()

	main(args)

""" python3 network_graph.py http://cs.marlboro.edu && dot network_graph.dot -Tpng -o output.png && eog -f output.png """
